﻿using System;
using System.IO.Ports;
using System.Windows.Forms;

namespace ArduinoAccessControlSetup
{
    public partial class Form1 : Form
    {
        private bool isLoggedIn = false;
        private string dataReceived = "";

        public Form1()
        {
            InitializeComponent();
            labelStatus.Text = "";
        }

        private void btnScan_Click(object sender, EventArgs e)
        {
            string[] ports = SerialPort.GetPortNames();
            listPorts.Items.Clear();
            btnConnect.Enabled = false;
            foreach (string port in ports)
            {
                listPorts.Items.Add(port);
            }
        }

        private void listPorts_SelectedIndexChanged(object sender, EventArgs e)
        {
            btnConnect.Enabled = true;
        }

        private void btnConnect_Click(object sender, EventArgs e)
        {

            try
            {
                if(serialPort.IsOpen)
                {
                    sendData("LOGOUT");
                    setLoggedState(false);
                    serialPort.Close();
                }
                else
                {
                    serialPort.PortName = listPorts.SelectedItem.ToString();
                    serialPort.Open();
                }
            }
            catch(Exception exception)
            {
                MessageBox.Show("ERROR: " + exception.Message);
            }

            if(serialPort.IsOpen)
            {
                btnConnect.Text = "Rozłącz";
                btnScan.Enabled = false;
                listPorts.Enabled = false;

                btnLogin.Enabled = true;
            }
            else
            {
                btnConnect.Text = "Połącz";
                btnScan.Enabled = true;
                listPorts.Enabled = true;

                btnLogin.Enabled = false;
                btnOpen.Enabled = false;
                btnAdd.Enabled = false;
                btnClear.Enabled = false;
            }
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            try
            {
                if(serialPort.IsOpen)
                {
                    sendData("LOGOUT");
                    serialPort.Close();
                }
            }
            catch (Exception exception)
            {
                MessageBox.Show("ERROR: " + exception.Message);
            }
        }

        private void btnLogin_Click(object sender, EventArgs e)
        {
            if(isLoggedIn)
            {
                sendData("LOGOUT");
            } else
            {
                sendData("LOGIN");
            }
        }

        private void btnAdd_Click(object sender, EventArgs e)
        {
            sendData("ADD");
        }

        private void btnClear_Click(object sender, EventArgs e)
        {
            sendData("CLEAR");
        }

        private void btnOpen_Click(object sender, EventArgs e)
        {
            sendData("OPEN");
        }

        private void sendData(string data)
        {
            try
            {
                serialPort.WriteLine(data);
            }
            catch (Exception exception)
            {
                MessageBox.Show("ERROR: " + exception.Message);
            }
        }

        private void serialPort_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {
            SerialPort sp = (SerialPort) sender;
            dataReceived = sp.ReadLine().Trim();
            Invoke(new EventHandler(handleReceivedData));
        }

        private void handleReceivedData(object sender, EventArgs e)
        {
            if("ALREADY_LOGGED_IN".Equals(dataReceived) || "LOGGED_IN".Equals(dataReceived))
            {
                setLoggedState(true);
                labelStatus.Text = "Zalogowano pomyślnie";
            }
            else if("LOGGED_OUT".Equals(dataReceived))
            {
                setLoggedState(false);
                labelStatus.Text = "Wylogowano pomyślnie";
            }
            else if("AUTH_FAILED".Equals(dataReceived) || "AUTH_TIMEOUT".Equals(dataReceived))
            {
                labelStatus.Text = "Autoryzacja nie powiodła się";
            }
            else if("AUTO_LOGGED_OUT".Equals(dataReceived))
            {
                setLoggedState(false);
                labelStatus.Text = "Wylogowano automatycznie";
            }
            else if("AUTH_WAITING".Equals(dataReceived))
            {
                labelStatus.Text = "Przyłóż kartę master do czytnika";
            }

            labelResponse.Text = dataReceived;
        }

        private void setLoggedState(bool isLoggedIn)
        {
            this.isLoggedIn = isLoggedIn;
            btnAdd.Enabled = isLoggedIn;
            btnClear.Enabled = isLoggedIn;
            btnOpen.Enabled = isLoggedIn;

            if (isLoggedIn)
                btnLogin.Text = "Wyloguj się";
            else
                btnLogin.Text = "Zaloguj się";
        }
    }
}
